#!/usr/bin/env python

################################################################
#
# Author: Carmelo Iannello <c.iannello [at] codices [dot] com>  
# Date: 2010-04-23
# 
################################################################

USAGE = '''Usage: %s infile outfile

infile should be a text file containing the CA server certificate
as extracted from C:\Program Files\CheckPoint\SecuRemote\database\usersc.C
Locate in usersc.C two lines that look like:
   :cert (5997eaa4492915d481aab1a91c7ae5 ... )
   :dn ("O=the.organization.you.want.to.connect.to")
look at the :dn field to choose among different server certificates you might
have in your configuration, then save the :cert line in a text file and
use it as the <infile> argument to this program.
It saves a <outfile>.der file and then converts it, using the command:

openssl x509 -inform DER -in <outfile>.der -outform PEM -out <outfile>

so you should have openssl installed and in your $PATH
The resulting <outfile> can be used as a "Server CA File" in IKE.
'''

import os
import sys
import struct

def main(args):
    fin = file(args[0])
    txt = fin.read()
    fin.close()
    
    try:
        start = txt.index(':cert')
        start = txt.index('(', start) + 1
    except ValueError:
        start = 0
    
    try:
        stop = txt.index(')', start)
    except ValueError:
        stop = len(txt)

    txt = txt[start:stop].strip(' \t\n\r')
    binary = []
    
    for i in range(0, len(txt), 2):
        hexstr = txt[i] + txt[i+1]
        binary.append( int(hexstr, 16) )
    
    binary.reverse()

    pemFname = args[1]
    derFname = args[1] + '.der'
    binfile = open(derFname, 'wb')
    for num in binary:
        data = struct.pack('B', num)
        binfile.write(data)
    
    binfile.close()
    
    print 'Created file :', derFname

    cmd = 'openssl x509 -inform DER -in "%s" -outform PEM -out "%s"' % (derFname, pemFname)
    print 'Now invoking the command: \n%s' % cmd
    
    xs = os.system(cmd)
    if xs != 0:
        print 'something went wrong: openssl exit code:', xs            
    else:
        print 'Done.'
##
    
if __name__ == '__main__':
    if len(sys.argv) < 3:
        print USAGE % sys.argv[0]
        sys.exit(1)
    main(sys.argv[1:])
